@@ -38,6 +38,10 @@
 #include <string.h>
 #include <unistd.h>
 
+/* Remember to grep for ->format_version in the code if you change this,
+ * most changes should be in dbus-gobject.c. */
+#define FORMAT_VERSION 1
+
 #define MARSHAL_PREFIX "dbus_glib_marshal_"
 
 typedef struct
@@ -246,7 +250,7 @@ compute_gsignature (MethodInfo *method, GType *rettype, GArray **params, GError
   g_set_error (error,
 	       DBUS_BINDING_TOOL_ERROR,
 	       DBUS_BINDING_TOOL_ERROR_UNSUPPORTED_CONVERSION,
-	       _("Unsupported conversion from D-BUS type %s to glib-genmarshal type"),
+	       "Unsupported conversion from D-BUS type %s to glib-genmarshal type",
 	       arg_type);
   return FALSE;
 }
@@ -466,14 +470,13 @@ generate_glue_toplevel (BaseInfo *base, DBusBindingToolCData *data, GError **err
 {
   GString *object_introspection_data_blob;
   GIOChannel *channel;
-  
-  channel = data->channel;
-  
-  object_introspection_data_blob = g_string_new_len ("", 0);
 
+  channel = data->channel;
+
+  object_introspection_data_blob = g_string_new_len ("", 0);
   data->blob = object_introspection_data_blob;
   data->count = 0;
-  
+
   data->signal_blob = g_string_new_len ("", 0);
   data->property_blob = g_string_new_len ("", 0);
 
@@ -490,10 +493,9 @@ generate_glue_toplevel (BaseInfo *base, DBusBindingToolCData *data, GError **err
   WRITE_OR_LOSE ("};\n\n");
   /* Information about the object. */
 
-  if (!write_printf_to_iochannel ("const DBusGObjectInfo dbus_glib_%s_object_info = {\n",
-                                  channel, error, data->prefix))
+  if (!write_printf_to_iochannel ("const DBusGObjectInfo dbus_glib_%s_object_info = {  %d,\n",
+                                  channel, error, data->prefix, FORMAT_VERSION))
     goto io_lose;
-  WRITE_OR_LOSE ("  0,\n");
   if (!write_printf_to_iochannel ("  dbus_glib_%s_methods,\n", channel, error, data->prefix))
     goto io_lose;
   if (!write_printf_to_iochannel ("  %d,\n", channel, error, data->count))
@@ -564,6 +566,7 @@ generate_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error)
           gboolean async = FALSE;
 	  GSList *args;
 	  gboolean found_retval = FALSE;
+          guint found_out_args = 0;
 
           method = (MethodInfo *) tmp->data;
 	  method_c_name = g_strdup (method_info_get_annotation (method, DBUS_GLIB_ANNOTATION_C_SYMBOL));
@@ -579,7 +582,11 @@ generate_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error)
 
           if (!write_printf_to_iochannel ("  { (GCallback) %s, ", channel, error,
 					  method_c_name))
-	    goto io_lose;
+            {
+	      g_free (method_c_name);
+              goto io_lose;
+            }
+          g_free (method_c_name);
 
           marshaller_name = compute_marshaller_name (method, data->prefix, error);
 	  if (!marshaller_name)
@@ -627,6 +634,7 @@ generate_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error)
 		  break;
 		case ARG_OUT:
 		  direction = 'O';
+                  found_out_args++;
 		  break;
 		case ARG_INVALID:
                 default:
@@ -686,6 +694,19 @@ generate_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error)
 				   interface_info_get_name (interface));
 		      return FALSE;
 		    }
+
+                  if (found_out_args != 1)
+                    {
+                      g_set_error (error,
+                          DBUS_BINDING_TOOL_ERROR,
+                          DBUS_BINDING_TOOL_ERROR_INVALID_ANNOTATION,
+                          "An output <arg> after the first cannot have the ReturnVal annotation, in argument \"%s\" of method \"%s\" of interface \"%s\"\n",
+                          arg_info_get_name (arg),
+                          method_info_get_name (method),
+                          interface_info_get_name (interface));
+                      return FALSE;
+                    }
+
 		  if (!strcmp ("", returnval_annotation))
 		    g_string_append_c (object_introspection_data_blob, 'R');
 		  else if (!strcmp ("error", returnval_annotation))
@@ -753,13 +774,36 @@ generate_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error)
       for (tmp = properties; tmp != NULL; tmp = g_slist_next (tmp))
         {
           PropertyInfo *prop;
-	  
-	  prop = tmp->data;
+          PropertyAccessFlags access_flags;
+          const char *access_string;
+          char *uscored;
 
-	  g_string_append (data->property_blob, interface_info_get_name (interface));
-	  g_string_append_c (data->property_blob, '\0');
-	  g_string_append (data->property_blob, property_info_get_name (prop));
-	  g_string_append_c (data->property_blob, '\0');
+          prop = tmp->data;
+
+          access_flags = property_info_get_access (prop);
+          if ((access_flags & PROPERTY_READ) && (access_flags & PROPERTY_WRITE))
+            access_string = "readwrite";
+          else if (access_flags & PROPERTY_READ)
+            access_string = "read";
+          else if (access_flags & PROPERTY_WRITE)
+            access_string = "write";
+          else
+            continue;
+
+          /* We append both in the blob so we have to malloc() less when processing
+           * properties */
+          uscored = _dbus_gutils_wincaps_to_uscore (property_info_get_name (prop));
+
+          g_string_append (data->property_blob, interface_info_get_name (interface));
+          g_string_append_c (data->property_blob, '\0');
+          g_string_append (data->property_blob, property_info_get_name (prop));
+          g_string_append_c (data->property_blob, '\0');
+          g_string_append (data->property_blob, uscored);
+          g_string_append_c (data->property_blob, '\0');
+          g_string_append (data->property_blob, access_string);
+          g_string_append_c (data->property_blob, '\0');
+
+          g_free (uscored);
 	}
     }
   return TRUE;
@@ -954,7 +998,7 @@ write_formal_parameters (InterfaceInfo *iface, MethodInfo *method, GIOChannel *c
 	  g_set_error (error,
 		       DBUS_BINDING_TOOL_ERROR,
 		       DBUS_BINDING_TOOL_ERROR_UNSUPPORTED_CONVERSION,
-		       _("Unsupported conversion from D-BUS type signature \"%s\" to glib C type in method \"%s\" of interface \"%s\""),
+		       "Unsupported conversion from D-BUS type signature \"%s\" to glib C type in method \"%s\" of interface \"%s\"",
 		       arg_info_get_type (arg),
 		       method_info_get_name (method),
 		       interface_info_get_name (iface));
@@ -1109,12 +1153,12 @@ static gboolean
 write_args_for_direction (InterfaceInfo *iface, MethodInfo *method, GIOChannel *channel, int direction, GError **error)
 {
   GSList *args;
+  char *type_lookup = NULL;
 
   for (args = method_info_get_args (method); args; args = args->next)
     {
       ArgInfo *arg;
       GType gtype;
-      char *type_lookup;
 
       arg = args->data;
 
@@ -1149,6 +1193,7 @@ write_args_for_direction (InterfaceInfo *iface, MethodInfo *method, GIOChannel *
 
   return TRUE;
  io_lose:
+  g_free (type_lookup);
   return FALSE;
 }
 
@@ -1217,7 +1262,7 @@ write_formal_declarations_for_direction (InterfaceInfo *iface, MethodInfo *metho
          g_set_error (error,
                       DBUS_BINDING_TOOL_ERROR,
                       DBUS_BINDING_TOOL_ERROR_UNSUPPORTED_CONVERSION,
-                      _("Unsupported conversion from D-BUS type signature \"%s\" to glib C type in method \"%s\" of interface \"%s\""),
+                      "Unsupported conversion from D-BUS type signature \"%s\" to glib C type in method \"%s\" of interface \"%s\"",
                       arg_info_get_type (arg),
                       method_info_get_name (method),
                       interface_info_get_name (iface));
@@ -1311,7 +1356,7 @@ write_formal_parameters_for_direction (InterfaceInfo *iface, MethodInfo *method,
 	  g_set_error (error,
 		       DBUS_BINDING_TOOL_ERROR,
 		       DBUS_BINDING_TOOL_ERROR_UNSUPPORTED_CONVERSION,
-		       _("Unsupported conversion from D-BUS type signature \"%s\" to glib C type in method \"%s\" of interface \"%s\""),
+		       "Unsupported conversion from D-BUS type signature \"%s\" to glib C type in method \"%s\" of interface \"%s\"",
 		       arg_info_get_type (arg),
 		       method_info_get_name (method),
 		       interface_info_get_name (iface));
@@ -1345,15 +1390,15 @@ write_formal_parameters_for_direction (InterfaceInfo *iface, MethodInfo *method,
 
 static gboolean
 write_typed_args_for_direction (InterfaceInfo *iface, MethodInfo *method, GIOChannel *channel, const int direction, GError **error)
- {
+{
   GSList *args;
+  char *type_lookup = NULL;
   
   for (args = method_info_get_args (method); args; args = args->next)
     {
       ArgInfo *arg;
       int dir;
       GType gtype;
-      const char *type_lookup;
       
       arg = args->data;
 
@@ -1366,10 +1411,12 @@ write_typed_args_for_direction (InterfaceInfo *iface, MethodInfo *method, GIOCha
       type_lookup = dbus_g_type_get_lookup_function (gtype);
 
       if (!write_printf_to_iochannel ("%s, &%s_%s, ", channel, error, type_lookup, direction == ARG_IN ? "IN" : "OUT", arg_info_get_name (arg)))
-          goto io_lose;
+        goto io_lose;
+      g_free (type_lookup);
     }
   return TRUE;
  io_lose:
+  g_free (type_lookup);
   return FALSE;
 }
 
@@ -1488,14 +1535,19 @@ generate_client_glue_list (GSList *list, DBusBindingToolCData *data, GError **er
 static gboolean
 generate_client_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error)
 {
+  char *iface_prefix;
+  char *method_c_name;
+  iface_prefix = NULL;
+  method_c_name = NULL;
+
   if (base_info_get_type (base) == INFO_TYPE_NODE)
     {
       if (!generate_client_glue_list (node_info_get_nodes ((NodeInfo *) base),
-				      data, error))
-	return FALSE;
+              data, error))
+        return FALSE;
       if (!generate_client_glue_list (node_info_get_interfaces ((NodeInfo *) base),
-				      data, error))
-	return FALSE;
+              data, error))
+        return FALSE;
     }
   else
     {
@@ -1503,7 +1555,6 @@ generate_client_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error
       InterfaceInfo *interface;
       GSList *methods;
       GSList *tmp;
-      char *iface_prefix;
       const char *interface_c_name;
 
       channel = data->channel;
@@ -1520,98 +1571,96 @@ generate_client_glue (BaseInfo *base, DBusBindingToolCData *data, GError **error
       }
 
       if (!write_printf_to_iochannel ("#ifndef DBUS_GLIB_CLIENT_WRAPPERS_%s\n"
-				      "#define DBUS_GLIB_CLIENT_WRAPPERS_%s\n\n",
-				      channel, error,
-				      iface_prefix, iface_prefix))
-	{
-	  g_free (iface_prefix);
-	  goto io_lose;
-	}
+              "#define DBUS_GLIB_CLIENT_WRAPPERS_%s\n\n",
+              channel, error,
+              iface_prefix, iface_prefix))
+        goto io_lose;
 
       for (tmp = methods; tmp != NULL; tmp = g_slist_next (tmp))
         {
-	  MethodInfo *method;
-	  char *method_c_name;
-	  gboolean is_noreply;
+          MethodInfo *method;
+          gboolean is_noreply;
 
           method = (MethodInfo *) tmp->data;
-	  method_c_name = g_strdup (method_info_get_annotation (method, DBUS_GLIB_ANNOTATION_CLIENT_C_SYMBOL));
+          method_c_name = g_strdup (method_info_get_annotation (method, DBUS_GLIB_ANNOTATION_CLIENT_C_SYMBOL));
           if (method_c_name == NULL)
-	    {
+            {
               method_c_name = compute_client_method_name (interface_c_name, method);
             }
 
-	  is_noreply = method_info_get_annotation (method, DBUS_GLIB_ANNOTATION_NOREPLY) != NULL;
+          is_noreply = method_info_get_annotation (method, DBUS_GLIB_ANNOTATION_NOREPLY) != NULL;
 
-	  if (data->ignore_unsupported && !check_supported_parameters (method))
-	    {
-	      g_warning ("Ignoring unsupported signature in method \"%s\" of interface \"%s\"\n",
-			 method_info_get_name (method),
-			 interface_info_get_name (interface));
-	      continue;
-	    }
+          if (data->ignore_unsupported && !check_supported_parameters (method))
+            {
+              g_warning ("Ignoring unsupported signature in method \"%s\" of interface \"%s\"\n",
+              method_info_get_name (method),
+              interface_info_get_name (interface));
+              g_free (method_c_name);
+              method_c_name = NULL;
+              continue;
+            }
 
 
-	  WRITE_OR_LOSE ("static\n#ifdef G_HAVE_INLINE\ninline\n#endif\ngboolean\n");
-	  if (!write_printf_to_iochannel ("%s (DBusGProxy *proxy", channel, error,
-					  method_c_name))
-	    goto io_lose;
-	  g_free (method_c_name);
+          WRITE_OR_LOSE ("static\n#ifdef G_HAVE_INLINE\ninline\n#endif\ngboolean\n");
+          if (!write_printf_to_iochannel ("%s (DBusGProxy *proxy", channel, error,
+                  method_c_name))
+            goto io_lose;
+          g_free (method_c_name);
+          method_c_name = NULL;
 
-	  if (!write_formal_parameters (interface, method, channel, error))
-	    goto io_lose;
+          if (!write_formal_parameters (interface, method, channel, error))
+            goto io_lose;
 
-	  WRITE_OR_LOSE (", GError **error)\n\n");
-	  
-	  WRITE_OR_LOSE ("{\n");
+          WRITE_OR_LOSE (", GError **error)\n\n");
 
-	  if (is_noreply) {
-	    if (!write_printf_to_iochannel ("  dbus_g_proxy_call_no_reply (proxy, \"%s\", ", channel, error,
-					    method_info_get_name (method)))
-	      goto io_lose;
-	    
-	    if (!write_args_for_direction (interface, method, channel, ARG_IN, error))
-	      goto io_lose;
-	    
-	    WRITE_OR_LOSE ("G_TYPE_INVALID, ");
-	    
-	    if (!write_args_for_direction (interface, method, channel, ARG_OUT, error))
-	      goto io_lose;
-	    
-	    WRITE_OR_LOSE ("G_TYPE_INVALID);\n");
-	    
-	    WRITE_OR_LOSE ("  return TRUE;\n}\n\n");
-	  } else {
-	    if (!write_printf_to_iochannel ("  return dbus_g_proxy_call (proxy, \"%s\", ", channel, error,
-					    method_info_get_name (method)))
-	      goto io_lose;
-	    
-	    WRITE_OR_LOSE ("error, ");
-	    
-	    if (!write_args_for_direction (interface, method, channel, ARG_IN, error))
-	      goto io_lose;
-	    
-	    WRITE_OR_LOSE ("G_TYPE_INVALID, ");
-	    
-	    if (!write_args_for_direction (interface, method, channel, ARG_OUT, error))
-	      goto io_lose;
-	    
-	    WRITE_OR_LOSE ("G_TYPE_INVALID);\n}\n\n");
-	  }
+          WRITE_OR_LOSE ("{\n");
 
-	  write_async_method_client (channel, interface, method, error);
-	}
+          if (is_noreply) {
+            if (!write_printf_to_iochannel ("  dbus_g_proxy_call_no_reply (proxy, \"%s\", ", channel, error,
+                    method_info_get_name (method)))
+              goto io_lose;
+
+            if (!write_args_for_direction (interface, method, channel, ARG_IN, error))
+              goto io_lose;
+
+            WRITE_OR_LOSE ("G_TYPE_INVALID, ");
+
+            if (!write_args_for_direction (interface, method, channel, ARG_OUT, error))
+              goto io_lose;
+
+            WRITE_OR_LOSE ("G_TYPE_INVALID);\n");
+
+            WRITE_OR_LOSE ("  return TRUE;\n}\n\n");
+          } else {
+            if (!write_printf_to_iochannel ("  return dbus_g_proxy_call (proxy, \"%s\", ", channel, error,
+                    method_info_get_name (method)))
+              goto io_lose;
+
+            WRITE_OR_LOSE ("error, ");
+
+            if (!write_args_for_direction (interface, method, channel, ARG_IN, error))
+              goto io_lose;
+
+            WRITE_OR_LOSE ("G_TYPE_INVALID, ");
+
+            if (!write_args_for_direction (interface, method, channel, ARG_OUT, error))
+              goto io_lose;
+
+            WRITE_OR_LOSE ("G_TYPE_INVALID);\n}\n\n");
+          }
+
+          write_async_method_client (channel, interface, method, error);
+        }
 
       if (!write_printf_to_iochannel ("#endif /* defined DBUS_GLIB_CLIENT_WRAPPERS_%s */\n\n", channel, error, iface_prefix))
-	{
-	  g_free (iface_prefix);
-	  goto io_lose;
-	}
+        goto io_lose;
 
       g_free (iface_prefix);
     }
   return TRUE;
  io_lose:
+  g_free (method_c_name);
+  g_free (iface_prefix);
   return FALSE;
 }
 
