@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -29,11 +29,6 @@
 #include "dbus-gobject.h"
 #include <string.h>
 
-#include <libintl.h>
-#define _(x) dgettext (GETTEXT_PACKAGE, x)
-#define N_(x) x
-
-
 /**
  * SECTION:dbus-gconnection
  * @short_description: DBus Connection
@@ -349,8 +344,7 @@ dbus_g_message_get_message (DBusGMessage *gmessage)
 
 #ifdef DBUS_BUILD_TESTS
 
-/**
- * @ingroup DBusGLibInternals
+/*
  * Unit test for general glib stuff
  * Returns: #TRUE on success.
  */
