@@ -33,20 +33,9 @@
 #include "dbus-gsignature.h"
 #include <string.h>
 
-#include <libintl.h>
-#define _(x) dgettext (GETTEXT_PACKAGE, x)
-#define N_(x) x
-
-/**
- * @defgroup DBusGLibInternals GLib bindings implementation details
- * @ingroup  DBusInternals
- * @brief Implementation details of GLib bindings
- *
- * @{
- */
-
-/**
+/*
  * DBusGMessageQueue:
+ *
  * A GSource subclass for dispatching DBusConnection messages.
  * We need this on top of the IO handlers, because sometimes
  * there are messages to dispatch queued up but no IO pending.
@@ -500,42 +489,30 @@ static ConnectionSetup*
 connection_setup_new_from_old (GMainContext    *context,
                                ConnectionSetup *old)
 {
-  GSList *tmp;
   ConnectionSetup *cs;
 
   g_assert (old->context != context);
   
   cs = connection_setup_new (context, old->connection);
   
-  tmp = old->ios;
-  while (tmp != NULL)
+  while (old->ios != NULL)
     {
-      IOHandler *handler = tmp->data;
+      IOHandler *handler = old->ios->data;
 
       connection_setup_add_watch (cs, handler->watch);
-      
-      tmp = tmp->next;
+      /* The old handler will be removed from old->ios as a side-effect */
     }
 
-  tmp = old->timeouts;
-  while (tmp != NULL)
+  while (old->timeouts != NULL)
     {
-      TimeoutHandler *handler = tmp->data;
+      TimeoutHandler *handler = old->timeouts->data;
 
       connection_setup_add_timeout (cs, handler->timeout);
-      
-      tmp = tmp->next;
     }
 
   return cs;
 }
 
-/** @} */ /* End of GLib bindings internals */
-
-/** @addtogroup DBusGLib
- * @{
- */
-
 /**
  * dbus_connection_setup_with_g_main:
  * @connection: the connection
@@ -656,7 +633,8 @@ dbus_server_setup_with_g_main (DBusServer   *server,
       cs = connection_setup_new_from_old (context, old_setup);
       
       /* Nuke the old setup */
-      dbus_server_set_data (server, server_slot, NULL, NULL);
+      if (!dbus_server_set_data (server, server_slot, NULL, NULL))
+        goto nomem;
       old_setup = NULL;
     }
 
@@ -809,12 +787,9 @@ dbus_g_bus_get_private (DBusBusType     type,
   return DBUS_G_CONNECTION_FROM_CONNECTION (connection);
 }
 
-/** @} */ /* end of public API */
-
 #ifdef DBUS_BUILD_TESTS
 
-/**
- * @ingroup DBusGLibInternals
+/*
  * Unit test for GLib main loop integration
  * Returns: #TRUE on success.
  */
