@@ -17,9 +17,12 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
+
+#include <config.h>
+
 #include "dbus-gparser.h"
 #include "dbus/dbus-glib-lowlevel.h"
 #include "dbus-gidl.h"
@@ -27,12 +30,6 @@
 #include "dbus/dbus-signature.h"
 #include <string.h>
 
-#include <libintl.h>
-#define _(x) gettext ((x))
-#define N_(x) x
-
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
 #define ELEMENT_IS(name) (strcmp (element_name, (name)) == 0)
 
 typedef struct
@@ -117,7 +114,7 @@ locate_attributes (const char  *element_name,
                   g_set_error (error,
                                G_MARKUP_ERROR,
                                G_MARKUP_ERROR_PARSE,
-                               _("Attribute \"%s\" repeated twice on the same <%s> element"),
+                               "Attribute \"%s\" repeated twice on the same <%s> element",
                                attrs[j].name, element_name);
                   retval = FALSE;
                   goto out;
@@ -138,7 +135,7 @@ locate_attributes (const char  *element_name,
               g_set_error (error,
                            G_MARKUP_ERROR,
                            G_MARKUP_ERROR_PARSE,
-                           _("Attribute \"%s\" is invalid on <%s> element in this context"),
+                           "Attribute \"%s\" is invalid on <%s> element in this context",
                            attribute_names[i], element_name);
               retval = FALSE;
               goto out;
@@ -164,7 +161,7 @@ check_no_attributes (const char  *element_name,
       g_set_error (error,
                    G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Attribute \"%s\" is invalid on <%s> element in this context"),
+                   "Attribute \"%s\" is invalid on <%s> element in this context",
                    attribute_names[0], element_name);
       return FALSE;
     }
@@ -260,7 +257,7 @@ parse_node (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -277,7 +274,7 @@ parse_node (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "name", element_name);
       return FALSE;
     }
@@ -287,7 +284,7 @@ parse_node (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute on <%s> element must be an absolute object path, \"%s\" not OK"),
+                   "\"%s\" attribute on <%s> element must be an absolute object path, \"%s\" not OK",
                    "name", element_name, name);
       return FALSE;
     }
@@ -297,7 +294,7 @@ parse_node (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute on <%s> element must not be an absolute object path, \"%s\" starts with /"),
+                   "\"%s\" attribute on <%s> element must not be an absolute object path, \"%s\" starts with /",
                    "name", element_name, name);
       return FALSE;
     }
@@ -337,7 +334,7 @@ parse_interface (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -353,7 +350,7 @@ parse_interface (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "name", element_name);
       return FALSE;
     }
@@ -389,7 +386,7 @@ parse_method (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -405,7 +402,7 @@ parse_method (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "name", element_name);
       return FALSE;
     }
@@ -439,7 +436,7 @@ parse_signal (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -455,7 +452,7 @@ parse_signal (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "name", element_name);
       return FALSE;
     }
@@ -509,7 +506,7 @@ parse_property (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -527,7 +524,7 @@ parse_property (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "name", element_name);
       return FALSE;
     }
@@ -536,7 +533,7 @@ parse_property (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "access", element_name);
       return FALSE;
     }
@@ -545,7 +542,7 @@ parse_property (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "type", element_name);
       return FALSE;
     }
@@ -564,7 +561,7 @@ parse_property (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("access=\"%s\" must have value readwrite, read, or write on %s\n"),
+                   "access=\"%s\" must have value readwrite, read, or write on %s\n",
                    access, element_name);
       return FALSE;
     }
@@ -600,7 +597,7 @@ parse_arg (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -620,7 +617,7 @@ parse_arg (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "type", element_name);
       return FALSE;
     }
@@ -649,11 +646,11 @@ parse_arg (Parser      *parser,
       if (parser->signal)
         g_set_error (error, G_MARKUP_ERROR,
                      G_MARKUP_ERROR_PARSE,
-                     _("Signals must have direction=\"out\" (just omit the direction attribute)"));
+                     "Signals must have direction=\"out\" (just omit the direction attribute)");
       else
         g_set_error (error, G_MARKUP_ERROR,
                      G_MARKUP_ERROR_PARSE,
-                     _("\"%s\" attribute on <%s> has value \"in\" or \"out\""),
+                     "\"%s\" attribute on <%s> has value \"in\" or \"out\"",
                      "direction", element_name);
       return FALSE;
     }
@@ -695,15 +692,14 @@ parse_annotation (Parser      *parser,
   const char *name;
   const char *value;
   
-  if (!(parser->method || parser->interface || parser->arg) || 
+  if (!(parser->method || parser->interface || parser->arg ||
+        parser->property || parser->signal) ||
       parser->node_stack == NULL ||
-      parser->signal ||
-      parser->property ||
       parser->in_annotation)
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("Can't put <%s> element here"),
+                   "Can't put <%s> element here",
                    element_name);
       return FALSE;      
     }
@@ -720,7 +716,7 @@ parse_annotation (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "name", element_name);
       return FALSE;
     }
@@ -728,7 +724,7 @@ parse_annotation (Parser      *parser,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_PARSE,
-                   _("\"%s\" attribute required on <%s> element "),
+                   "\"%s\" attribute required on <%s> element ",
                    "value", element_name);
       return FALSE;
     }
@@ -807,7 +803,7 @@ parser_start_element (Parser      *parser,
         {
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_PARSE,
-                       _("Element <%s> not recognized"),
+                       "Element <%s> not recognized",
                        element_name);
           return FALSE;
         }
@@ -902,5 +898,3 @@ parser_get_nodes (Parser *parser)
 {
   return parser->result;
 }
-
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
