@@ -21,6 +21,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
+
+#include <config.h>
+
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include <dbus/dbus-signature.h>
@@ -32,30 +35,24 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gobject/gvaluecollector.h>
+#include <gio/gio.h>
 
 #define DBUS_G_PROXY_CALL_TO_ID(x) (GPOINTER_TO_UINT(x))
 #define DBUS_G_PROXY_ID_TO_CALL(x) (GUINT_TO_POINTER(x))
 #define DBUS_G_PROXY_GET_PRIVATE(o)  \
        (G_TYPE_INSTANCE_GET_PRIVATE ((o), DBUS_TYPE_G_PROXY, DBusGProxyPrivate))
 
-
-/**
- * @addtogroup DBusGLibInternals
- *
- * @{
- */
-
-/**
- * DBusGProxyManager typedef
- */
+static void oom (void) G_GNUC_NORETURN;
+static void
+oom (void)
+{
+  g_error ("no memory");
+}
 
 typedef struct _DBusGProxyManager DBusGProxyManager;
 
 typedef struct _DBusGProxyPrivate DBusGProxyPrivate;
 
-/**
- * Internals of DBusGProxy
- */
 struct _DBusGProxyPrivate
 {
   DBusGProxyManager *manager; /**< Proxy manager */
@@ -117,7 +114,7 @@ static gboolean dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 						GType              first_arg_type,
 						va_list            args);
 
-/**
+/*
  * A list of proxies with a given name+path+interface, used to
  * route incoming signals.
  */
@@ -132,7 +129,7 @@ typedef struct
   
 } DBusGProxyList;
 
-/**
+/*
  * DBusGProxyManager's primary task is to route signals to the proxies
  * those signals are emitted on. In order to do this it also has to
  * track the owners of the names proxies are bound to.
@@ -147,8 +144,14 @@ struct _DBusGProxyManager
 
   GHashTable *proxy_lists; /**< Hash used to route incoming signals
                             *   and iterate over proxies
+                            *   tristring -> DBusGProxyList
+                            */
+  GHashTable *owner_match_rules; /**< Hash to keep track of match rules of
+                                  * NameOwnerChanged.
+                                  * gchar *name -> guint *refcount
                             */
   GHashTable *owner_names; /**< Hash to keep track of mapping from
+                            *   char *    -> GSList of DBusGProxyNameOwnerInfo
 			    *   base name -> [name,name,...] for proxies which
 			    *   are for names.
 			    */
@@ -260,6 +263,16 @@ dbus_g_proxy_manager_unref (DBusGProxyManager *manager)
 
         }
 
+      if (manager->owner_match_rules)
+        {
+	  /* Since we destroyed all proxies, none can be tracking
+	   * name owners
+	   */
+          g_assert (g_hash_table_size (manager->owner_match_rules) == 0);
+          g_hash_table_destroy (manager->owner_match_rules);
+          manager->owner_match_rules = NULL;
+        }
+
       if (manager->owner_names)
 	{
 	  /* Since we destroyed all proxies, none can be tracking
@@ -485,18 +498,13 @@ g_proxy_get_signal_match_rule (DBusGProxy *proxy)
 }
 
 static char *
-g_proxy_get_owner_match_rule (DBusGProxy *proxy)
+get_owner_match_rule (const gchar *name)
 {
-  DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
-  if (priv->name) 
-    {
       return g_strdup_printf ("type='signal',sender='" DBUS_SERVICE_DBUS
         "',path='" DBUS_PATH_DBUS
         "',interface='" DBUS_INTERFACE_DBUS
         "',member='NameOwnerChanged'"
-        ",arg0='%s'", priv->name);
-    }
-  return NULL;
+        ",arg0='%s'", name);
 }
 
 typedef struct
@@ -910,6 +918,13 @@ get_name_owner (DBusConnection     *connection,
 }
 
 
+static void
+guint_slice_free (gpointer data)
+{
+  g_slice_free (guint, data);
+}
+
+
 static void
 dbus_g_proxy_manager_register (DBusGProxyManager *manager,
                                DBusGProxy        *proxy)
@@ -922,6 +937,7 @@ dbus_g_proxy_manager_register (DBusGProxyManager *manager,
   if (manager->proxy_lists == NULL)
     {
       g_assert (manager->owner_names == NULL);
+      g_assert (manager->owner_match_rules == NULL);
 
       list = NULL;
       manager->proxy_lists = g_hash_table_new_full (tristring_hash,
@@ -932,6 +948,10 @@ dbus_g_proxy_manager_register (DBusGProxyManager *manager,
                                                     g_str_equal,
                                                     g_free,
                                                     NULL);
+      manager->owner_match_rules = g_hash_table_new_full (g_str_hash,
+                                                          g_str_equal,
+                                                          g_free,
+                                                          guint_slice_free);
     }
   else
     {
@@ -958,21 +978,37 @@ dbus_g_proxy_manager_register (DBusGProxyManager *manager,
        * but only if the server is a message bus,
        * not if it's a peer.
        */
-       char *rule;
+      char *rule;
+      guint *refcount;
+
+      rule = g_proxy_get_signal_match_rule (proxy);
+      /* We don't check for errors; it's not like anyone would handle them, and
+       * we don't want a round trip here.
+       */
+      dbus_bus_add_match (manager->connection, rule, NULL);
+      g_free (rule);
        
-       rule = g_proxy_get_signal_match_rule (proxy);
-       /* We don't check for errors; it's not like anyone would handle them, and
-        * we don't want a round trip here.
-        */
-       dbus_bus_add_match (manager->connection,
-			   rule, NULL);
-       g_free (rule);
-       
-       rule = g_proxy_get_owner_match_rule (proxy);
-       if (rule)
-         dbus_bus_add_match (manager->connection,
-                             rule, NULL);
-       g_free (rule);
+      refcount = g_hash_table_lookup (manager->owner_match_rules, priv->name);
+
+      if (refcount != NULL)
+        {
+          g_assert (*refcount != 0);
+          g_assert (*refcount < G_MAXUINT);
+          (*refcount)++;
+        }
+      else
+        {
+          char *rule;
+          rule = get_owner_match_rule (priv->name);
+          dbus_bus_add_match (manager->connection,
+                              rule, NULL);
+          g_free (rule);
+
+          refcount = g_slice_new (guint);
+          *refcount = 1;
+          g_hash_table_insert (manager->owner_match_rules,
+                               g_strdup (priv->name), refcount);
+        }
     }
 
   g_assert (g_slist_find (list->proxies, proxy) == NULL);
@@ -1073,17 +1109,27 @@ dbus_g_proxy_manager_unregister (DBusGProxyManager *manager,
       char *rule;
       g_hash_table_remove (manager->proxy_lists,
                            tri);
-      list = NULL;
 
       rule = g_proxy_get_signal_match_rule (proxy);
       dbus_bus_remove_match (manager->connection,
                              rule, NULL);
       g_free (rule);
-      rule = g_proxy_get_owner_match_rule (proxy);
-      if (rule)
-        dbus_bus_remove_match (manager->connection,
-                               rule, NULL);
-      g_free (rule);      
+
+      if (priv->name)
+        {
+          guint *refcount;
+          refcount = g_hash_table_lookup (manager->owner_match_rules, priv->name);
+          (*refcount)--;
+
+          if (*refcount == 0)
+            {
+              rule = get_owner_match_rule (priv->name);
+              dbus_bus_remove_match (manager->connection,
+                                     rule, NULL);
+              g_free (rule);
+              g_hash_table_remove (manager->owner_match_rules, priv->name);
+            }
+        }
     }
   
   if (g_hash_table_size (manager->proxy_lists) == 0)
@@ -1092,6 +1138,12 @@ dbus_g_proxy_manager_unregister (DBusGProxyManager *manager,
       manager->proxy_lists = NULL;
     }
 
+  if (g_hash_table_size (manager->owner_match_rules) == 0)
+    {
+      g_hash_table_destroy (manager->owner_match_rules);
+      manager->owner_match_rules = NULL;
+    }
+
   g_free (tri);
       
   UNLOCK_MANAGER (manager);
@@ -1840,12 +1892,6 @@ manager_begin_bus_call (DBusGProxyManager    *manager,
   return call;
 }
 
-/** @} End of DBusGLibInternals */
-
-/** @addtogroup DBusGLib
- * @{
- */
-
 /**
  * SECTION:dbus-gproxy
  * @short_description: DBus Proxy
@@ -1855,7 +1901,7 @@ manager_begin_bus_call (DBusGProxyManager    *manager,
  * A #DBusGProxy is a boxed type abstracting a #DBusProxy.
  */
 
-/**
+/*
  * dbus_g_proxy_get_type:
  * Standard GObject get_type() function for DBusGProxy.
  *
@@ -1943,9 +1989,9 @@ dbus_g_proxy_new_for_name (DBusGConnection *connection,
                            const char      *interface_name)
 {
   g_return_val_if_fail (connection != NULL, NULL);
-  g_return_val_if_fail (name != NULL, NULL);
-  g_return_val_if_fail (path_name != NULL, NULL);
-  g_return_val_if_fail (interface_name != NULL, NULL);
+  g_return_val_if_fail (g_dbus_is_name (name), NULL);
+  g_return_val_if_fail (g_variant_is_object_path (path_name), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (interface_name), NULL);
 
   return dbus_g_proxy_new (connection, name,
 			   path_name, interface_name);
@@ -1987,9 +2033,9 @@ dbus_g_proxy_new_for_name_owner (DBusGConnection          *connection,
   char *unique_name;
 
   g_return_val_if_fail (connection != NULL, NULL);
-  g_return_val_if_fail (name != NULL, NULL);
-  g_return_val_if_fail (path_name != NULL, NULL);
-  g_return_val_if_fail (interface_name != NULL, NULL);
+  g_return_val_if_fail (g_dbus_is_name (name), NULL);
+  g_return_val_if_fail (g_variant_is_object_path (path_name), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (interface_name), NULL);
 
   if (!(unique_name = get_name_owner (DBUS_CONNECTION_FROM_G_CONNECTION (connection), name, error)))
     return NULL;
@@ -2018,7 +2064,10 @@ dbus_g_proxy_new_from_proxy (DBusGProxy        *proxy,
 {
   DBusGProxyPrivate *priv;
 
-  g_return_val_if_fail (proxy != NULL, NULL);
+  g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), NULL);
+  g_return_val_if_fail (path == NULL || g_variant_is_object_path (path), NULL);
+  g_return_val_if_fail (interface == NULL ||
+      g_dbus_is_interface_name (interface), NULL);
 
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
   
@@ -2054,8 +2103,8 @@ dbus_g_proxy_new_for_peer (DBusGConnection          *connection,
   DBusGProxy *proxy;
   
   g_return_val_if_fail (connection != NULL, NULL);
-  g_return_val_if_fail (path_name != NULL, NULL);
-  g_return_val_if_fail (interface_name != NULL, NULL);
+  g_return_val_if_fail (g_variant_is_object_path (path_name), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (interface_name), NULL);
 
   proxy = dbus_g_proxy_new (connection, NULL,
                             path_name, interface_name);
@@ -2074,6 +2123,9 @@ dbus_g_proxy_new_for_peer (DBusGConnection          *connection,
  * unique connection name will be returned. If you created it
  * with dbus_g_proxy_new_for_peer() then #NULL will be returned.
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * Returns: the bus name the proxy sends messages to
  */
 const char*
@@ -2095,6 +2147,9 @@ dbus_g_proxy_get_bus_name (DBusGProxy        *proxy)
  *
  * Gets the object interface proxy is bound to (may be #NULL in some cases).
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * Returns: an object interface 
  */
 const char*
@@ -2116,12 +2171,20 @@ dbus_g_proxy_get_interface (DBusGProxy        *proxy)
  * @interface_name: an object interface 
  *
  * Sets the object interface proxy is bound to
+ *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
  */
 void
 dbus_g_proxy_set_interface (DBusGProxy        *proxy,
 			    const char        *interface_name)
 {
   DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
+
+  g_return_if_fail (DBUS_IS_G_PROXY (proxy));
+  g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
+  g_return_if_fail (g_dbus_is_interface_name (interface_name));
+
   /* FIXME - need to unregister when we switch interface for now
    * later should support idea of unset interface
    */
@@ -2136,6 +2199,9 @@ dbus_g_proxy_set_interface (DBusGProxy        *proxy,
  * Gets the path this proxy is bound to
  * @proxy: the proxy
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * Returns: an object path
  */
 const char*
@@ -2166,7 +2232,7 @@ dbus_g_proxy_marshal_args_to_message (DBusGProxy  *proxy,
                                           priv->interface,
                                           method);
   if (message == NULL)
-    goto oom;
+    return NULL;
 
   dbus_message_iter_init_append (message, &msgiter);
   for (i = 0; i < args->n_values; i++)
@@ -2176,11 +2242,19 @@ dbus_g_proxy_marshal_args_to_message (DBusGProxy  *proxy,
       gvalue = g_value_array_get_nth (args, i);
 
       if (!_dbus_gvalue_marshal (&msgiter, gvalue))
-	g_assert_not_reached ();
+        {
+          /* This is a programming error by the caller, most likely */
+          gchar *contents = g_strdup_value_contents (gvalue);
+
+          g_critical ("Could not marshal argument %u for %s: type %s, value %s",
+              i, method, G_VALUE_TYPE_NAME (gvalue), contents);
+          g_free (contents);
+          dbus_message_unref (message);
+          return NULL;
+        }
     }
+
   return message;
- oom:
-  return NULL;
 }
 
 static guint
@@ -2201,14 +2275,17 @@ dbus_g_proxy_begin_call_internal (DBusGProxy          *proxy,
   pending = NULL;
 
   message = dbus_g_proxy_marshal_args_to_message (proxy, method, args);
+
+  /* can only happen on a programming error or OOM; we already critical'd */
   if (!message)
-    goto oom;
+    return 0;
 
   if (!dbus_connection_send_with_reply (priv->manager->connection,
                                         message,
                                         &pending,
                                         timeout))
-    goto oom;
+    oom ();
+
   dbus_message_unref (message);
   
   /* If we got a NULL pending, that means the connection was disconnected,
@@ -2236,9 +2313,6 @@ dbus_g_proxy_begin_call_internal (DBusGProxy          *proxy,
   g_hash_table_insert (priv->pending_calls, GUINT_TO_POINTER (call_id), pending);
 
   return call_id;
- oom:
-  g_error ("Out of memory");
-  return 0;
 }
 
 static gboolean
@@ -2297,7 +2371,7 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 	    {
 	      g_set_error (error, DBUS_GERROR,
 			   DBUS_GERROR_INVALID_ARGS,
-			   _("Too few arguments in reply"));
+			   "Too few arguments in reply");
 	      goto out;
 	    }
 
@@ -2316,7 +2390,7 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 		  g_set_error (error,
 			       DBUS_GERROR,
 			       DBUS_GERROR_INVALID_ARGS,
-			       _("Couldn't convert argument, expected \"%s\""),
+			       "Couldn't convert argument, expected \"%s\"",
 			       g_type_name (valtype));
 		  goto out;
 		}
@@ -2350,7 +2424,7 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
 	{
 	  g_set_error (error, DBUS_GERROR,
 		       DBUS_GERROR_INVALID_ARGS,
-		       _("Too many arguments in reply; expected %d, got %d"),
+		       "Too many arguments in reply; expected %d, got %d",
 		       n_retvals_processed, over);
 	  goto out;
 	}
@@ -2424,6 +2498,9 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
  * will be invoked; you can then collect the results of the call
  * (which may be an error, or a reply), use dbus_g_proxy_end_call().
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * TODO this particular function shouldn't die on out of memory,
  * since you should be able to do a call with large arguments.
  * 
@@ -2445,6 +2522,7 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
   
   g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), NULL);
   g_return_val_if_fail (!DBUS_G_PROXY_DESTROYED (proxy), NULL);
+  g_return_val_if_fail (g_dbus_is_member_name (method), NULL);
 
   va_start (args, first_arg_type);
 
@@ -2466,7 +2544,7 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
  * @notify: callback to be invoked when method returns
  * @user_data: user data passed to callback
  * @destroy: function called to destroy user_data
- * @timeout: specify the timeout in milliseconds
+ * @timeout: the timeout in milliseconds, or -1 to use a default
  * @first_arg_type: type of the first argument
  *
  * Asynchronously invokes a method on a remote interface. The method
@@ -2477,6 +2555,9 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
  * will be invoked; you can then collect the results of the call
  * (which may be an error, or a reply), use dbus_g_proxy_end_call().
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * TODO this particular function shouldn't die on out of memory,
  * since you should be able to do a call with large arguments.
  *
@@ -2498,6 +2579,8 @@ dbus_g_proxy_begin_call_with_timeout (DBusGProxy          *proxy,
 
   g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), NULL);
   g_return_val_if_fail (!DBUS_G_PROXY_DESTROYED (proxy), NULL);
+  g_return_val_if_fail (g_dbus_is_member_name (method), NULL);
+  g_return_val_if_fail (timeout >= 0 || timeout == -1, NULL);
 
   va_start (args, first_arg_type);
 
@@ -2544,6 +2627,8 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
   gboolean ret;
   va_list args;
 
+  g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), FALSE);
+
   va_start (args, first_arg_type);
 
   ret = dbus_g_proxy_end_call_internal (proxy, GPOINTER_TO_UINT (call), error, first_arg_type, args);
@@ -2567,6 +2652,9 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
  * output values, followed by a second G_TYPE_INVALID.  Note that  
  * this means you must always specify G_TYPE_INVALID twice.
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * Returns: #FALSE if an error is set, #TRUE otherwise.
  */
 gboolean
@@ -2604,7 +2692,7 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
     {
       g_set_error (error, DBUS_GERROR,
 		   DBUS_GERROR_FAILED,
-		   _("Disconnection or out-of-memory"));
+		   "Disconnection or out-of-memory");
       ret = FALSE;
     }
 
@@ -2617,7 +2705,7 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
  * dbus_g_proxy_call_with_timeout:
  * @proxy: a proxy for a remote interface
  * @method: method to invoke
- * @timeout: specify the timeout in milliseconds
+ * @timeout: the timeout in milliseconds, or -1 to use a default
  * @error: return location for an error
  * @first_arg_type: type of first "in" argument
  *
@@ -2628,6 +2716,9 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
  * output values, followed by a second G_TYPE_INVALID.  Note that
  * this means you must always specify G_TYPE_INVALID twice.
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * Returns: #FALSE if an error is set, #TRUE otherwise.
  */
 gboolean
@@ -2645,6 +2736,8 @@ dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
 
   g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), FALSE);
   g_return_val_if_fail (!DBUS_G_PROXY_DESTROYED (proxy), FALSE);
+  g_return_val_if_fail (g_dbus_is_member_name (method), FALSE);
+  g_return_val_if_fail (timeout >= 0 || timeout == -1, FALSE);
 
   va_start (args, first_arg_type);
 
@@ -2671,6 +2764,9 @@ dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
  * Sends a method call message as with dbus_g_proxy_begin_call(), but
  * does not ask for a reply or allow you to receive one.
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * TODO: this particular function shouldn't die on out of memory,
  * since you should be able to do a call with large arguments.
  */
@@ -2686,6 +2782,7 @@ dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
   DBusGProxyPrivate *priv;
   
   g_return_if_fail (DBUS_IS_G_PROXY (proxy));
+  g_return_if_fail (g_dbus_is_member_name (method));
   g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
 
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
@@ -2698,20 +2795,18 @@ dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
   g_value_array_free (in_args);
   va_end (args);
 
+  /* can only happen on a programming error or OOM; we already critical'd */
   if (!message)
-    goto oom;
+    return;
 
   dbus_message_set_no_reply (message, TRUE);
 
   if (!dbus_connection_send (priv->manager->connection,
                              message,
                              NULL))
-    goto oom;
+    oom ();
+
   dbus_message_unref (message);
-  return;
-  
- oom:
-  g_error ("Out of memory");
 }
 
 /**
@@ -2723,6 +2818,9 @@ dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
  * initiated with dbus_g_proxy_begin_call().  This function
  * may not be used on pending calls that have already been
  * ended with dbus_g_proxy_end_call.
+ *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
  */
 void
 dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
@@ -2765,6 +2863,9 @@ dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
  *
  * This function adds a reference to the message, so the caller
  * still owns its original reference.
+ *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
  */
 void
 dbus_g_proxy_send (DBusGProxy          *proxy,
@@ -2810,9 +2911,15 @@ array_free_all (gpointer array)
  * @signal_name: the name of the signal
  * @first_type: the first argument type, or G_TYPE_INVALID if none
  *
- * Specifies the argument signature of a signal;.only necessary
- * if the remote object does not support introspection.  The arguments
- * specified are the GLib types expected.
+ * Specifies the argument signature of a D-Bus signal. When the signal is
+ * emitted by the remote object, if the GTypes corresponding to its arguments'
+ * types do not match the types given here, the signal will be ignored.
+ *
+ * It is an error to add the same @signal_name to the same @proxy more than
+ * once, even if the argument types given are the same.
+ *
+ * It is also an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
  */
 void
 dbus_g_proxy_add_signal  (DBusGProxy        *proxy,
@@ -2829,8 +2936,8 @@ dbus_g_proxy_add_signal  (DBusGProxy        *proxy,
 
   g_return_if_fail (DBUS_IS_G_PROXY (proxy));
   g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
-  g_return_if_fail (signal_name != NULL);
-  
+  g_return_if_fail (g_dbus_is_member_name (signal_name));
+
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
 
   name = create_signal_name (priv->interface, signal_name);
@@ -2874,6 +2981,9 @@ dbus_g_proxy_add_signal  (DBusGProxy        *proxy,
  * Connect a signal handler to a proxy for a remote interface.  When
  * the remote interface emits the specified signal, the proxy will
  * emit a corresponding GLib signal.
+ *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
  */
 void
 dbus_g_proxy_connect_signal (DBusGProxy             *proxy,
@@ -2889,7 +2999,7 @@ dbus_g_proxy_connect_signal (DBusGProxy             *proxy,
 
   g_return_if_fail (DBUS_IS_G_PROXY (proxy));
   g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
-  g_return_if_fail (signal_name != NULL);
+  g_return_if_fail (g_dbus_is_member_name (signal_name));
   g_return_if_fail (handler != NULL);
   
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
@@ -2925,6 +3035,9 @@ dbus_g_proxy_connect_signal (DBusGProxy             *proxy,
  *
  * Disconnect all signal handlers from a proxy that match the given
  * criteria.
+ *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
  */
 void
 dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
@@ -2938,7 +3051,7 @@ dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
   
   g_return_if_fail (DBUS_IS_G_PROXY (proxy));
   g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
-  g_return_if_fail (signal_name != NULL);
+  g_return_if_fail (g_dbus_is_member_name (signal_name));
   g_return_if_fail (handler != NULL);
 
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
@@ -2969,10 +3082,12 @@ dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
 /**
  * dbus_g_proxy_set_default_timeout:
  * @proxy: a proxy for a remote interface
- * @timeout: specify the timeout in milliseconds
+ * @timeout: the timeout in milliseconds, or -1 to reset to the libdbus default
  *
  * Sets the default timeout to use for a proxy. This timeout will be
- * used in calls where the timeout is not specified.
+ * used in calls where the timeout is not specified, or is specified to be -1.
+ * If this timeout is also set to -1, libdbus will use a reasonable default
+ * value.
  *
  * This is useful for long-running operations that takes longer than
  * the default timeout (which is a on the order of magnitude of tens
@@ -2981,6 +3096,9 @@ dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
  * (e.g. immediately) and emits a signal when the operation terminates
  * (though beware of leaking information with/in the signal return value).
  *
+ * It is an error to call this method on a proxy that has emitted
+ * the #DBusGProxy::destroy signal.
+ *
  * Since: 0.75
  */
 void
@@ -2991,27 +3109,8 @@ dbus_g_proxy_set_default_timeout (DBusGProxy        *proxy,
 
   g_return_if_fail (DBUS_IS_G_PROXY (proxy));
   g_return_if_fail (!DBUS_G_PROXY_DESTROYED (proxy));
+  g_return_if_fail (timeout >= 0 || timeout == -1);
 
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
   priv->default_timeout = timeout;
 }
-
-
-/** @} End of DBusGLib public */
-
-#ifdef DBUS_BUILD_TESTS
-
-/**
- * @ingroup DBusGLibInternals
- * Unit test for GLib proxy functions
- * Returns: #TRUE on success.
- */
-gboolean
-_dbus_g_proxy_test (void)
-{
-  
-  
-  return TRUE;
-}
-
-#endif /* DBUS_BUILD_TESTS */
